/* Cria montanhas aleatorias com variacoes de cores */



#include <conio.h>
#include <string.h>
#include <bios.h>
#include <stdio.h>
#include <stdlib.h>

#include "defs.h"

#include "arquivo.h"

#include "grvd.h"

#include "apl.h"



/* construtor - inicia gerador de numeros aleatorios,...
**********************************************************/
CApl::CApl ( char* szArq )
{
   // inicia modo grafico
   m_poGr = new CGrVd ( TRUE );

   // inicia gerador de numeros aleatorios
   srandom ( (int) biostime ( 0, 0 ) );

   // limpa matriz de dados sobre cores
   memset ( (void*) m_achSup, 0, RESX * RESY );

   // define nome do arquivo para salvar imagem
   strcpy ( m_szArq, szArq );
}



/* destrutor - termina modo grafico
**********************************************************/
CApl::~CApl ()
{
   delete m_poGr;
}



/* Primeira tecnica de geracao de cores:
 Define uma area do grafico e a divide em 4 subareas.
As faixas de cores sao definidas por progresao aritmetica.
Usa recursao com condicao de parada: area menor ou igual a 1.
nXMin, nYMin - canto esquerdo superior da area
nXMax, nYMax - canto direito inferior da area
nBase - altura do nivel
nFaixa - intervalo para os possiveis novos valores
**********************************************************/
void CApl::mDefineArea ( INT nXMin, INT nYMin, INT nXMax, INT nYMax, INT nBase, INT nFaixa )
{
   // define cor da area
   nBase = nBase + ( random () % nFaixa - nFaixa / 2 );

   // define pixel da area se jah nao foi definido
   if ( m_achSup [ nYMax ] [ nXMax ] == 0 ) {

      m_achSup [ nYMax ] [ nXMax ] = nBase + 128;
      // verifica se estah dentro da tela
      if ( nXMax < 320 && nYMax < 200 )
         m_poGr->mAtrVdPixel ( nXMax, nYMax, nBase + 128 );
   }

   // condisao de parada
   if ( nXMax - nXMin <= 1 && nYMax - nYMin <= 1 )
      return;

   // calcula meio da area
   INT
      nXMed = ( nXMax + 1 + nXMin ) / 2,
      nYMed = ( nYMax + 1 + nYMin ) / 2;

   // define proxima faixa de valores
   nFaixa -= 31;

   // divide area em mais 4 subareas
   mDefineArea ( nXMed, nYMed, nXMax, nYMax, nBase, nFaixa );
   mDefineArea ( nXMin, nYMed, nXMed, nYMax, nBase, nFaixa );
   mDefineArea ( nXMin, nYMin, nXMed, nYMed, nBase, nFaixa );
   mDefineArea ( nXMed, nYMin, nXMax, nYMed, nBase, nFaixa );
}



/* Define cor de um ponto - usado pela segunda tecnica de geracao de cores
**********************************************************/
INT CApl::mRetCor ( INT nSomaC, INT nDist, INT nDiv )
{
   if ( nDiv == 0 || nDist == 0 )
      return 0;

   INT
      nCor = ( ( nSomaC + nDist - ( random () % nDist ) ) / nDiv ) % 245;

   return nCor;
}



/* Segunda tecnica de geracao de cores:
 Define uma area do grafico e a divide em 4 subareas
 Se baseia no conteudo do arquivo VOXEL.PAS de GRFXFX2.ZIP.
As faixas de cores sao definidas por media das cores anteriores.
Usa recursao com condicao de parada: area menor ou igual a 1.
nXMin, nYMin - canto esquerdo superior da area
nXMax, nYMax - canto direito inferior da area
**********************************************************/
void CApl::mDefineArea2 ( INT nXMin, INT nYMin, INT nXMax, INT nYMax )
{
   // condisao de parada
   if ( nXMax - nXMin <= 1 && nYMax - nYMin <= 1 )
      return;

   // cor dos pontos extremos da area
   INT
      nCor1 = m_achSup [ nYMin ] [ nXMin ],
      nCor2 = m_achSup [ nYMax ] [ nXMin ],
      nCor3 = m_achSup [ nYMin ] [ nXMax ],
      nCor4 = m_achSup [ nYMax ] [ nXMax ];

   // calcula meio da area
   INT
      nXMed = ( nXMax + 1 + nXMin ) / 2,
      nYMed = ( nYMax + 1 + nYMin ) / 2;

   // valor dependente da dimensao da area
   INT
      nDist = (INT) ( (float) 5 * ( nXMax - nXMin + nYMax - nYMin ) / 3 );

   // marca pontos do meio da area se jah nao foram marcados
   if ( m_achSup [ nYMin ] [ nXMed ] == 0 ) {

      // define cor
      INT
         nCor = mRetCor ( nCor1 + nCor3, nDist, 2 /*7*/ );

      // marca ponto
      m_achSup [ nYMin ] [ nXMed ] = nCor;
      // verifica se estah dentro da tela
      if ( nXMed < 320 && nYMin < 200 )
         m_poGr->mAtrVdPixel ( nXMed, nYMin, nCor );
   }
   if ( m_achSup [ nYMed ] [ nXMin ] == 0 ) {

      // define cor
      INT
         nCor = mRetCor ( nCor1 + nCor2, nDist, 2 /*7*/ );

      // marca ponto
      m_achSup [ nYMed ] [ nXMin ] = nCor;
      // verifica se estah dentro da tela
      if ( nXMin < 320 && nYMed < 200 )
         m_poGr->mAtrVdPixel ( nXMin, nYMed, nCor );
   }
   if ( m_achSup [ nYMed ] [ nXMax ] == 0 ) {

      // define cor
      INT
         nCor = mRetCor ( nCor3 + nCor4, nDist, 2 /*7*/ );

      // marca ponto
      m_achSup [ nYMin ] [ nXMed ] = nCor;
      // verifica se estah dentro da tela
      if ( nXMax < 320 && nYMed < 200 )
         m_poGr->mAtrVdPixel ( nXMax, nYMed, nCor );
   }
   if ( m_achSup [ nYMax ] [ nXMed ] == 0 ) {

      // define cor
      INT
         nCor = mRetCor ( nCor2 + nCor4, nDist, 2 /*7*/ );

      // marca ponto
      m_achSup [ nYMin ] [ nXMed ] = nCor;
      // verifica se estah dentro da tela
      if ( nXMed < 320 && nYMax < 200 )
         m_poGr->mAtrVdPixel ( nXMed, nYMax, nCor );
   }

   // define cor do ponto medio
   INT
      nCor = mRetCor ( nCor1 + nCor2 + nCor3 + nCor4, nDist, 4 /*2*/ );
   // marca ponto medio
   m_achSup [ nYMed ] [ nXMed ] = nCor;
   // verifica se estah dentro da tela
   if ( nXMed < 320 && nYMed < 200 )
      m_poGr->mAtrVdPixel ( nXMed, nYMed, nCor );

   // divide area em mais 4 subareas
   mDefineArea2 ( nXMin, nYMin, nXMed, nYMed );
   mDefineArea2 ( nXMin, nYMed, nXMed, nYMax );
   mDefineArea2 ( nXMed, nYMin, nXMax, nYMed );
   mDefineArea2 ( nXMed, nYMed, nXMax, nYMax );
}



/* Inicia criacao dos graficos
**********************************************************/
void CApl::mExec ()
{
   // algoritmo 1:
   mDefineArea ( 0, 0, RESX - 1, RESY - 1, 0, 256 );

	getch ();

   // algoritmo 2:
   mDefineArea2 ( 0, 0, RESX - 1, RESY - 1 );

   getch ();

   // verifica se foi definido algum arquivo, se sim salva
   if ( m_szArq [ 0 ] != 0 ) {

      // varre imagem transformando para faixa de cores utilizada
      char
         *pachSupArq = new char [ RESY * RESX ];
      for ( INT ncY = 0; ncY < RESY; ncY ++ )
         for ( INT ncX = 0; ncX < RESX; ncX ++ )
            * ( pachSupArq + ncY * RESX + ncX ) = /*11 - 11 * */ m_achSup [ ncY ] [ ncX ] /*/ 256*/;

      // abre arquivo
      CArquivo
         oArq ( m_szArq, ".raw", "wb" );
      
      // grava
      oArq.mAtrMem ( RESX * RESY, (void*) pachSupArq );

      // desaloca memoria
      delete pachSupArq;
   }
}



main ( INT nNumArgs, char *szArgs [] )
{
   // verifica se foi passado nome de arquivo
   char
      szArq [ 500 ];
   if ( nNumArgs == 1 )
      szArq [ 0 ] = 0;
   else
      strcpy ( szArq, szArgs [ 1 ] );

   CApl
      *poApl = new CApl ( szArq );

   // verifica se criou aplicacao, se sim executa
   if ( poApl != NULL ) {

      poApl -> mExec ();
      delete poApl;
   }
   else
      printf ( MSG_IMPOSSIVELEXECUTAR );

   // exibe mensagem final
   printf ( MSG_TITULO );
}
